#!/bin/bash
# (C) Copyright 2007-2013 Hewlett Packard Development Company, L.P.
# Version A.12.00.00

################################################################################
# HA SAMBA Toolkit Main Script
################################################################################
# This script contains a list of internal-use variables and functions that
# support start/stop of a SAMBA server and its monitor process. To be a part
# of a ServiceGuard package, this script has to be placed in the package
# directory. This script can be called by the Toolkit Interface Script or
# directly by the Package Control Script.
#
# Parameters:
#       Action: either "start" or "stop"
#
# Exit Code:
#       1 - when a failure occurs during the "start" or "stop" process.
#       0 - normal exit.
#
# REVISION HISTORY:
#     Version      mm/dd/yyyy   Changes
#     A.03.00      07/13/2006   1) Removed the temporary file smbtkt.tmp after 
#                                  its use
#                               2) Removed RETRY_TIMES and MONITOR_INTERVAL as 
#                                  arguments in calling hasmb.mon
#                               3) Changed Signal names to Signal numbers in 
#                                  kill command
#     A.04.00                   1) Made modular compliant
#     A.04.02                   1) Added code to accept 'monitor' as a parameter
#                                  which will call hasmb.mon script
#     A.04.02.01   04/30/2012   1) Added check for the attribute
#                                  MONITOR_INTERVAL.
#     A.12.00.00   09/15/2013   1) Removed references to hasmb.conf and TKIT_DIR
################################################################################
 
# Note: Do not modify this file
 
# Include the SG PATHs 

. /etc/cmcluster.conf 
 
PATH=/bin:/sbin:/usr/bin:/usr/sbin:/bin
 
PKG_DIR=${0%/*}
if [ $PKG_DIR = ${0##*/} ]; then
    PKG_DIR=`pwd`
fi
CONF_DIR=$PKG_DIR

# Use the SAMBA default configuration file if the base configuration file 
# is not defined
if [ -z $BASE_CONF_FILE ]; then
    BASE_CONF_FILE="/etc/samba/smb.conf"
fi

if [ ! -f $BASE_CONF_FILE ]; then
    echo "ERROR: The SAMBA base configuration file $BASE_CONF_FILE does not exist."
    exit 1
fi

# Assigning default value 30 to MONITOR_INTERVAL.
if [ -z $MONITOR_INTERVAL ]; then
    MONITOR_INTERVAL=30
fi

# The ACTION is either "start" or "stop"
ACTION=$1

# A lock file to be used for process synchronization. This synchronization 
# prevents multiple SAMBA package main scripts accessing a single global 
# SAMBA package state file simultaneously. 
PKG_LOCK_FILE="/var/lock/smb_pkg.lock"

# File path of the global SAMBA package state file
PKG_STAT_FILE="/var/run/.smb_pkg.stat"

# File path of the final SAMBA configuraiton file. This single file contains 
# the combination of BASE_CONF_FILE and SUB_CONF_FILE. 
SMB_RUN_CONF_FILE="/etc/samba/smb_run.conf"


# START OF RUNNING FUNCTIONS

# This function creates a target SAMBA configuration file for this SAMBA 
# package
# Parameter: $1 = start | stop 
# Return value:	0: success; 1: fail; 2: multiple instances are running 
function make_conf_file 
{
    local ret_value=0
    local sub_conf_list
    local sub_conf
    
    [ ! -e $PKG_STAT_FILE ] && touch $PKG_STAT_FILE

    if [ ! -z $SUB_CONF_FILE ]; then
        if [ -f $SUB_CONF_FILE ] ; then
            sed "/${SUB_CONF_FILE//\//\\/}/d" $PKG_STAT_FILE > /tmp/smbtkit.tmp
            cp -f /tmp/smbtkit.tmp $PKG_STAT_FILE 
	    rm -f /tmp/smbtkit.tmp
            if [ "$1" = "start" ]; then
                echo $SUB_CONF_FILE >> $PKG_STAT_FILE
                echo $SUB_CONF_FILE
            fi
        else
            echo "ERROR: $SUB_CONF_FILE file does not exist."
            return 1
        fi
    fi

    sub_conf_list=`cat $PKG_STAT_FILE`

    cp -f $BASE_CONF_FILE $SMB_RUN_CONF_FILE
    for sub_conf in $sub_conf_list
    do
        echo "include = $sub_conf" >> $SMB_RUN_CONF_FILE
    done 
    
    if [ "$1" = "stop" ]; then
        if [ ${#sub_conf_list} -ne 0 ]; then
            return 2
        fi
    fi

    return $ret_value
}
 

# This function starts the SAMBA daemons
#
function start_server
{
    echo "$(date '+%b %e %T') - Node \"$(hostname)\": Starting SAMBA daemons."

    local ret_code=0
    local ret_val

    # Lock
    lockfile -l 15 $PKG_LOCK_FILE
    check_return $? 1
    
    # build a current state samba configuration file
    make_conf_file start
    ret_val=$?
    if (( $ret_val == 1)); then
        rm -f $PKG_LOCK_FILE
        check_return $ret_val 1
    fi

    pid=`pidofproc smbd`
    if [ -z "$pid" ]; then
        echo -e "\tStarting smbd daemon..."
        smbd -D -s $SMB_RUN_CONF_FILE
    else 
        echo -e "\tReloading smbd daemon..."
        kill -1 $pid 
    fi
    [ $? -ne 0 ] && ret_code=1

    pid=`pidofproc nmbd`
    if [ -z "$pid" ]; then
        echo -e "\tStarting nmbd daemon..."
        nmbd -D -s $SMB_RUN_CONF_FILE
    else 
        echo -e "\tReloading nmbd daemon..."
        kill -1 $pid 
    fi
    
    [ $? -ne 0 ] && ret_code=1

    # Unlock
    rm -f $PKG_LOCK_FILE

    check_return $ret_code 1
}


# This function kills SAMBA daemons.
#
function stop_server
{
    echo "$(date '+%b %e %T') - Node \"$(hostname)\": Stopping SAMBA daemons"
  
    local ret_val
    local ret_code=0
    local run_status="single"

    # Lock
    lockfile -l 15 $PKG_LOCK_FILE
    check_return $? 2
 
    # Build a current state samba configuration file
    make_conf_file stop
    ret_val=$?
    if (( $ret_val == 1 )); then  
        # Unlock
        rm -f $PKG_LOCK_FILE
        check_return $ret_val 2
    fi   

    # The make_conf_file will return 2 if any other SAMBA instance is running
    [ $ret_val -eq 2 ] && run_status="multiple"

    if [ "$run_status" = "single" ]; then
        echo -e "\tStopping smbd daemon..."
        killproc smbd -15
    else
        echo -e "\tReloading smbd daemon..."
        killproc smbd -1
    fi
    [ $? -ne 0 ] && ret_code=1

    if [ "$run_status" = "single" ]; then 
        echo -e "\tStopping nmbd daemon..."
        killproc nmbd -15
    else
        echo -e "\tReloading nmbd daemon..."
        killproc nmbd -1
    fi
    [ $? -ne 0 ] && ret_code=1

    # Unlock
    rm -f $PKG_LOCK_FILE

    if [ $ret_code -ne 0 ]; then
        echo "WARNING: Function stop_server: Failed to stop/reload SAMBA daemons"
    fi

}



################################################################################
# functions

# Find all pid of a process
#
function pidofproc
{
    pidof $1
}  

# Verify if pid(s)  are still running
verifypid() {
    while [ "$1" ]; do
       [ -d /proc/$1 ] && return 0
       shift
    done
    return 1
}

# Function that kills a named process
# Parameter:	$1: process name
#		$2: kill signal
function killproc 
{
    local ret_value=0
    local kill_sig="-15"
    [ ! -z $2 ] && kill_sig=$2 
    pidlist=`pidofproc $1`

    if [ ! -z "$pidlist" ]; then
        if [ "$kill_sig" = "-1" ] ; then
            if verifypid $pidlist; then
                kill -1 $pidlist
                ret_value=$?
            fi
        else
            if verifypid $pidlist; then
                kill -15 $pidlist
                usleep 100000
                if verifypid $pidlist && sleep 1 &&
                   verifypid $pidlist && sleep 3 &&
                   verifypid $pidlist ; then
                     kill $kill_sig $pidlist
                     usleep 100000
                fi
            fi
            verifypid $pidlist
            if [ $? -eq 0 ]; then
                ret_value=1
            fi
        fi
    fi

    return $ret_value
}


# Check return value of functions and exit.
# Parameter:    $1: exit code of a previous command
#               $2: function ID
function check_return
{
    if (( $1 == 1 ))
    then
	case $2 in
	    1)
		echo "ERROR: Function start_server: Failed to start a SAMBA daemons"
                stop_server
                exit 1
                ;;
	    2)
                echo "ERROR: Function stop_server: Failed to stop SAMBA daemons"
                exit 1
                ;;
	    *)
		echo "ERROR: Failed, unknown error."
		;;
	esac
    fi
}


# Script Start  

exit_value=0

# Test to see if we are being called to run the package, or stop the package, or monitor it.
if [ "$ACTION" = "start" ]; then

    start_server

elif [ "$ACTION" = "monitor" ]; then
    echo "$(date '+%b %e %T') - Node \"$(hostname)\": Starting hasmb.mon service"
    # Monitor script parameter list:
    # hasmb.mon  [-i monitor_interval] [-r retry_times]
    . $PKG_DIR/hasmb.mon -i $MONITOR_INTERVAL -r $RETRY_TIMES


elif [ "$ACTION" = "stop" ]; then

    
    stop_server
fi

exit $exit_value

 
